#include <iostream>
#include <string>

using namespace std;

class Player
{
public:
    Player(const string& name = "");
    string GetName() const;
    Player* GetNext() const;
    void SetNext(Player* next);
    int GetPosition() const;
    void SetPosition(int position);

private:
    string m_Name;
    Player* m_pNext;
    int m_Position;
};

Player::Player(const string& name) :
    m_Name(name),
    m_pNext(0),
    m_Position(0)
{}

string Player::GetName() const
{
    return m_Name;
}

Player* Player::GetNext() const
{
    return m_pNext;
}

void Player::SetNext(Player* next)
{
    m_pNext = next;
}

int Player::GetPosition() const
{
    return m_Position;
}

void Player::SetPosition(int position)
{
    m_Position = position;
}

class Lobby
{
    friend ostream& operator<<(ostream& os, const Lobby& aLobby);

public:
    Lobby();
    ~Lobby();
    void AddPlayer();
    void RemovePlayer();
    void Clear();
    void Search();
    void Alphabetize();

private:
    Player* m_pHead;
};

Lobby::Lobby() :
    m_pHead(0)
{}

Lobby::~Lobby()
{
    Clear();
}

void Lobby::AddPlayer()
{
    // Create a new player node
    cout << "Please enter the name of the new player: ";
    string name;
    cin >> name;
    Player* pNewPlayer = new Player(name);

    // If the list is empty, make the head of the list this new player
    if (m_pHead == 0)
    {
        m_pHead = pNewPlayer;
    }
    // Otherwise, find the end of the list and add the player there
    else
    {
        Player* pIter = m_pHead;
        int position = 1;
        while (pIter->GetNext() != 0)
        {
            pIter = pIter->GetNext();
            position++;
        }
        pIter->SetNext(pNewPlayer);
        pNewPlayer->SetPosition(position);
    }
}

void Lobby::RemovePlayer()
{
    string NameS;
    Player* pTemp = NULL;
    bool found = false;

    if (m_pHead == 0)
    {
        cout << "The game lobby is empty. No one to remove!\n";
    }
    else
    {
        cout << "Enter the name of the player you want to remove: ";
        cin >> NameS;

        Player* pPrev = NULL;
        Player* pIter = m_pHead;

        while (pIter != 0)
        {
            if (pIter->GetName() == NameS)
            {
                found = true;
                pTemp = pIter;

                if (pPrev == NULL)
                {
                    m_pHead = pIter->GetNext();
                }
                else
                {
                    pPrev->SetNext(pIter->GetNext());
                }

                int removedPosition = pTemp->GetPosition();
                delete pTemp;

                // Update positions for remaining players
                pIter = m_pHead;
                while (pIter != 0)
                {
                    pIter->SetPosition(pIter->GetPosition() - 1);
                    pIter = pIter->GetNext();
                }

                cout << NameS << " has been removed from the lobby.\n";
                break;
            }
            else
            {
                pPrev = pIter;
                pIter = pIter->GetNext();
            }
        }

        if (!found)
        {
            cout << NameS << " is not in the lobby.\n";
        }
    }
}

void Lobby::Clear()
{
    while (m_pHead != 0)
    {
        RemovePlayer();
    }
}

void Lobby::Search()
{
    string NameS;
    Player* pTemp = NULL;
    bool found = false;

    if (m_pHead == 0)
    {
        cout << "\nThere are no players in the lobby" << endl;
    }
    else
    {
        cout << "\nWhat name would you like to search for: ";
        cin >> NameS;

        Player* pIter = m_pHead;
        int position = 1;
        while (pIter != 0)
        {
            if (pIter->GetName() == NameS)
            {
                found = true;
                pTemp = pIter;
                break;
            }
            else
            {
                pIter = pIter->GetNext();
                position++;
            }
        }

        if (found)
        {
            cout << "The Player does exist and is in the lobby at position " << pTemp->GetPosition() + 1 << ".\n";
        }
        else
        {
            cout << "This Player is not in the lobby.\n";
        }
    }
}
void Lobby::Alphabetize()
{
    if (m_pHead == 0 || m_pHead->GetNext() == nullptr)
    {
        // No need to sort an empty list or a list with only one player
        return;
    }
    else {
        Player* pIter = m_pHead;
        Player* pIter2 = pIter->GetNext();
        Player* pIterCopy = pIter;
        Player* pIter2Copy = pIter2;


        while (pIter != 0) {
            if (pIter->GetName() > pIter2->GetName()) {
                pIter = pIter2Copy;
                pIter->SetNext(pIterCopy);

                pIter = pIter->GetNext();
                pIter2 = pIter2->GetNext();
            }
            else {
                break;
            }
        }
    }
}

ostream& operator<<(ostream& os, const Lobby& aLobby)
{
    Player* pIter = aLobby.m_pHead;

    os << "\nHere's who's in the game lobby:\n";
    if (pIter == 0)
    {
        os << "The lobby is empty.\n";
    }
    else
    {
        while (pIter != 0)
        {
            os << pIter->GetName() << endl;
            pIter = pIter->GetNext();
        }
    }

    return os;
}

int main()
{
    Lobby myLobby;
    int choice;

    do
    {
        cout << myLobby;
        cout << "\nGAME LOBBY\n";
        cout << "0 - Exit the program.\n";
        cout << "1 - Add a player to the lobby.\n";
        cout << "2 - Remove a player from the lobby.\n";
        cout << "3 - Clear the lobby.\n";
        cout << "4 - Search Player.\n";
        cout << "5 - Alphabetize the Lobby.\n";
        cout << endl << "Enter choice: ";
        cin >> choice;

        switch (choice)
        {
        case 0: cout << "Good-bye.\n"; break;
        case 1: myLobby.AddPlayer(); break;
        case 2: myLobby.RemovePlayer(); break;
        case 3: myLobby.Clear(); break;
        case 4: myLobby.Search(); break;
        case 5: myLobby.Alphabetize(); break;
        default: cout << "That was not a valid choice.\n";
        }
    } while (choice != 0);

    return 0;
}
